/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.commands.operations.ObjectUndoContext;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IStatusLineManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.IPartListener2;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPartReference;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.actions.ActionFactory;
import org.eclipse.ui.forms.IManagedForm;
import org.eclipse.ui.forms.ManagedForm;
import org.eclipse.ui.forms.widgets.ScrolledForm;
import org.eclipse.ui.forms.widgets.TableWrapLayout;
import org.eclipse.ui.operations.RedoActionHandler;
import org.eclipse.ui.operations.UndoActionHandler;
import org.eclipse.ui.part.EditorPart;
import org.eclipse.ui.views.properties.IPropertySheetPage;
import org.eclipse.ui.views.properties.IPropertySource;
import org.eclipse.ui.views.properties.PropertySheetPage;

import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.controller.PropertySheetController;
import eu.etaxonomy.taxeditor.controller.GlobalController;
import eu.etaxonomy.taxeditor.propertysheet.CustomSortPropertySheetEntry;

/**
 * The abstract editor for displaying a category of <code>Taxon</code> data, corresponding
 * to the tabs ("Name", "Descriptive", etc.) at the bottom of a <code>Taxon</code> view. Implements
 * <code>IAdaptable</code> in order to display properties of the objects whose UI elements have focus. 
 * <p>
 * Implementing classes can choose to show an object in the property sheet when the
 * <code>AbstractTaxonEditor</code> gets focus, by passing the object to the method
 * <code>setDefaultPropertySheetObject</code>, for instance, in the method<code>init</code>.
 * </p>
 * @author p.ciardelli
 * @created 10.09.2008
 * @version 1.0
 */
public abstract class AbstractTaxonEditor extends EditorPart implements
		IAdaptable {
	private static final Logger logger = Logger
			.getLogger(AbstractTaxonEditor.class);

	private Taxon taxon;
	
	/**
	 * When this <code>EditorPart</code> gets focus, the data structure of 
	 * <code>defaultPropertyObject</code> is displayed in the property sheet.
	 */
	
	protected IManagedForm managedForm;
	protected ScrolledForm scrolledForm; 
	protected Composite parent;
	protected ISelectionProvider provider;
	protected IUndoContext undoContext;

	protected IHasPropertySource selectedObject;
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.EditorPart#doSave(org.eclipse.core.runtime.IProgressMonitor)
	 */
	public void doSave(IProgressMonitor monitor) {}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.EditorPart#doSaveAs()
	 */
	public void doSaveAs() {}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.EditorPart#init(org.eclipse.ui.IEditorSite, org.eclipse.ui.IEditorInput)
	 */
	public void init(IEditorSite site, IEditorInput input)
			throws PartInitException {
		
		if (!(input instanceof IEditorInput))
			throw new PartInitException(
				"Invalid Input: Must be IFileEditorInput");
		
		if (input.getAdapter(Taxon.class) != null) {
			taxon = (Taxon) input.getAdapter(Taxon.class);
		} else {
			throw new PartInitException(
				"Invalid Input: Taxon cannot be null");
		}

		setSite(site);
		setInput(input);
		
		this.provider = new SimpleSelectionProvider();
		this.getSite().setSelectionProvider(provider);
	}

	private void initUndoContext() {		
		undoContext = new ObjectUndoContext(this); 
		// getEditorSite returns multipageeditor, NOT this
		// UndoActionHandler expects a IWorkbenchPartSite

//		UndoActionHandler undoHandler = new UndoActionHandler(getEditorSite(), undoContext);
//		IActionBars actionBars = getEditorSite().getActionBars();
		UndoActionHandler undoHandler = new UndoActionHandler(getSite(), undoContext);
		RedoActionHandler redoHandler = new RedoActionHandler(getSite(), undoContext);
		IActionBars actionBars = getEditorSite().getActionBars();
		actionBars.setGlobalActionHandler(ActionFactory.UNDO.getId(), undoHandler);
		actionBars.setGlobalActionHandler(ActionFactory.REDO.getId(), redoHandler);
	}
	
	public IUndoContext getUndoContext() {
		return undoContext;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.WorkbenchPart#setFocus()
	 */
	public void setFocus() {
//		setSelection(selectedObject);
	}
	
	/**
	 * If there is a default property sheet object with a corresponding property source class, 
	 * display it in the property sheet. Otherwise, empty the property sheet with an empty
	 * <code>StructuredSelection</code>.
	 * 
	 * @param selectedObject
	 */
	protected void setSelection(IHasPropertySource selectedObject) {
				
		// Unpaint last selection - last selection will only be unpainted
		//	when something else on this page is selected
		if (this.selectedObject instanceof EditorGroupedComposite) {
			((EditorGroupedComposite) this.selectedObject).unpaintBorder();
		}
		
		// Set the selection to this editor's selected object
		this.selectedObject = selectedObject;
		
		// Get the selection's property source, pass it to the selection provider
		IPropertySource propertySource = null;
		if (selectedObject != null) {
			propertySource = selectedObject.getPropertySource();
		}
		if (propertySource == null) {
			provider.setSelection(new StructuredSelection());
		} else {
			provider.setSelection(new StructuredSelection(propertySource));
		}
		
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.EditorPart#isDirty()
	 */
	public boolean isDirty() {
		return false;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.EditorPart#isSaveAsAllowed()
	 */
	public boolean isSaveAsAllowed() {
		return false;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.WorkbenchPart#createPartControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createPartControl(Composite composite) {

		initUndoContext();
		
		createManagedForm(composite);
	}
	
	protected void createManagedForm(Composite composite) {
		
		managedForm = new ManagedForm(composite) {
			public void dirtyStateChanged() {
				firePropertyChange(PROP_DIRTY);
			}
			public boolean setInput(Object input) {
				if (input instanceof IHasPropertySource) {
					setSelection((IHasPropertySource)input);
				}
				return super.setInput(input); 
			} 
		};
		scrolledForm = managedForm.getForm();
		parent = scrolledForm.getBody();
				
		Taxon taxon = getTaxon();
		parent.setData(taxon);
		
		parent.setLayout(new TableWrapLayout());		
		parent.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_WHITE));
	}
	
	@SuppressWarnings("unchecked")
	public Object getAdapter(Class type) {
	    if (type == IPropertySheetPage.class) {
	    	
	        PropertySheetPage page = new PropertySheetPage() {
	            public void makeContributions(IMenuManager menuManager,
	                    IToolBarManager toolBarManager, IStatusLineManager statusLineManager) {
	            	super.makeContributions(menuManager, toolBarManager, statusLineManager);
	            
	            	// Remove "Show categories", "Show advanced properties", "Restore default value"
	            	toolBarManager.removeAll();
	            	menuManager.removeAll();
	            }
	            
	            public Control getControl() {
	            	Control control = super.getControl();
	            	
	            	// Save the property sheet tree for easy access as needed
	            	if (!control.isDisposed()) {
	            		if (control instanceof Tree) {
	            			PropertySheetController.setPropertySheetTree((Tree) control);
	            		}
	            	}
	            	return control;
	            }
	        };	        
	        
	        // Try out PartListener to intercept selections
	        IWorkbenchPage activePage = GlobalController.getActivePage();
//	        IWorkbenchPart active = activePage.getActivePart();
	        IPartListener2 partListener = new IPartListener2() {

				@Override
				public void partActivated(IWorkbenchPartReference partRef) {
					// Fires for every part
//					logger.warn(partRef.getTitle());
				}

				@Override
				public void partBroughtToTop(IWorkbenchPartReference partRef) {
					// TODO Auto-generated method stub
					
				}

				@Override
				public void partClosed(IWorkbenchPartReference partRef) {
					// TODO Auto-generated method stub
					
				}

				@Override
				public void partDeactivated(IWorkbenchPartReference partRef) {
					// TODO Auto-generated method stub
					
				}

				@Override
				public void partHidden(IWorkbenchPartReference partRef) {
					// TODO Auto-generated method stub
					
				}

				@Override
				public void partInputChanged(IWorkbenchPartReference partRef) {
//					logger.warn("part input changed");
				}

				@Override
				public void partOpened(IWorkbenchPartReference partRef) {
					// TODO Auto-generated method stub
					
				}

				@Override
				public void partVisible(IWorkbenchPartReference partRef) {
					// TODO Auto-generated method stub
					
				}
	        	
	        };
	        activePage.addPartListener(partListener);

	        PropertySheetController.setPropertySheetPage(page);
	        
	        CustomSortPropertySheetEntry entry = new CustomSortPropertySheetEntry();
	        page.setRootEntry(entry);
	        page.refresh();
	        
	        return page;
	    }
	    return super.getAdapter(type);
	}
	
	protected Taxon getTaxon() {
		return taxon;
	}
	
	public void setDirty() {
		managedForm.dirtyStateChanged();
	}
}
