/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.editor.description;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.Assert;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.text.Document;
import org.eclipse.jface.text.source.SourceViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.FocusAdapter;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.forms.IManagedForm;
import org.eclipse.ui.forms.widgets.TableWrapData;
import org.eclipse.ui.views.properties.IPropertySource;

import eu.etaxonomy.cdm.model.common.Language;
import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.description.TextData;
import eu.etaxonomy.taxeditor.ITaxEditorConstants;
import eu.etaxonomy.taxeditor.TaxEditorPlugin;
import eu.etaxonomy.taxeditor.actions.ui.description.DeleteElementCompositeAction;
import eu.etaxonomy.taxeditor.editor.ContextMenu;
import eu.etaxonomy.taxeditor.editor.EditorGroupedComposite;
import eu.etaxonomy.taxeditor.editor.LineWrapSupport;
import eu.etaxonomy.taxeditor.editor.ParseListener;
import eu.etaxonomy.taxeditor.propertysheet.description.DescriptionElementPropertySource;

/**
 * @author p.ciardelli
 * @created 02.06.2008
 * @version 1.0
 */
public class DescriptionElementComposite extends EditorGroupedComposite {
	private static final Logger logger = Logger.getLogger(DescriptionElementComposite.class);

	/**
	 * ************ COMPOSITE TYPES ************
	 */
	public static final String TEXT_DATA = "text_data";
	public static final String COMMON_TAXON_NAME = "common_taxon_name";
	public static final String DISTRIBUTION = "distribution";

	/**
	 * ************ INDENTATIONS ************
	 */
//	public static final int ACCEPTED_INDENT = 0;

	/**
	 * ************ FONTS ************
	 */
	public static final Font ELEMENT_FONT = TaxEditorPlugin.getDefault()
			.getFont(ITaxEditorConstants.SYNONYM_FONT);

	/**
	 * ************ ICONS ************
	 */
	public static final Image BLACK_SQUARE_ICON = TaxEditorPlugin.getDefault()
			.getImage(ITaxEditorConstants.BLACK_SQUARE_ICON);

	/**
	 * ************ TRANSFORMATIONS ************
	 */
//	public static final String ADD_GROUP_BASIONYM = "add_group_basionym";

	/**
	 * ************ MENU ACTIONS ************
	 */
	public Action CHANGE_TAXON_TO_SYNONYM_ACTION;

//	private DescriptionElementBase element;

	private static final String EMPTY_NAME_PROMPT = "Click to add descriptive element text";

	/**
	 * The constructor for a <code>DescriptionElementComposite</code>. Takes a parent 
	 * <code>Composite</code> on which to create itself, and an <code>IManagedForm</code> for 
	 * <code>Composite</code> life cycle methods, i.e. drawing borders, creating other 
	 * <code>Composite</code>s, creating line wrap support, etc.
	 * 
	 * @param parent
	 * @param managedForm
	 */
	public DescriptionElementComposite(Composite parent, IManagedForm managedForm, 
			DescriptionElementBase element) {
		super(parent, managedForm);
		
		createTextViewer();
		createBorderSupport();
		createLineWrapSupport();

		setData(element);
		transform(null);
		
		createEmptyViewerPrompt(EMPTY_NAME_PROMPT);
		setFocus();
	}

	protected void createLineWrapSupport() {
		new LineWrapSupport(getTextViewer(), managedForm);
	}

	/**
	 * All cosmetic - non-data-related, i.e. icons, fonts, etc. -
	 * transformations take place in this method.
	 * 
	 * @param transformation
	 */
	public void transform(String transformation) {
		setDraggable(true);
		setFont(ELEMENT_FONT);
		setIcon(BLACK_SQUARE_ICON);
		
		createMenu();

		managedForm.getForm().getBody().layout();
	}

	/**
	 * Override of Composite.setData() which passes data along to
	 * DescriptionElementComposite's TextViewer where appropriate.
	 * 
	 * @see org.eclipse.swt.widgets.Widget#setData(java.lang.Object)
	 */
	public void setData(Object data) {
		super.setData(data);
		
		Assert.isTrue((data instanceof DescriptionElementBase), 
				"DescriptionElementComposite's data field must contain a DescriptionElementBase object");	
		
		DescriptionElementBase element = (DescriptionElementBase) data;

		String text = null;
		if (element instanceof TextData) {
			text = ((TextData) element).getText(Language.DEFAULT());
		}
		if (text == null) {
			text = "";
		}
		
		getTextViewer().getTextWidget().setText(text);
	}

	private void createTextViewer() {
		this.textViewer = new SourceViewer(this, null, SWT.WRAP | SWT.MULTI | SWT.RESIZE);
		this.textViewer.setDocument(new Document(""));
		this.textViewer.getTextWidget().setLayoutData(new TableWrapData(TableWrapData.FILL_GRAB));
		this.textViewer.getTextWidget().addModifyListener(new ParseListener() {

			@Override
			public void parse(String text) {
				Assert.isTrue((getData() instanceof DescriptionElementBase), 
						"DescriptionElementComposite's data field must contain a DescriptionElementBase object");
				
				DescriptionElementBase element = (DescriptionElementBase) getData();
				
				if (element instanceof TextData) {
					((TextData) element).putText(text, Language.DEFAULT());
				}
				
				setDirty(true);
			}
		});
		this.textViewer.getTextWidget().addFocusListener(new FocusAdapter() {
			public void focusGained(FocusEvent e) {
				setFocus();
			}			
		});
	}

	private void createMenu() {
		ContextMenu contextMenu = new ContextMenu(getDraggableLabel());
		
		// Delete this description element
		Action deleteElementAction = new DeleteElementCompositeAction(
				this, getTaxon());
		contextMenu.addAction(deleteElementAction);
		
		// Add new text 
	}
	
	@Override
	public void dispose() {
		
		Composite featureComponent = this.getParent();
		
		super.dispose();
		
		// If feature only has 1 child that is not DescriptionElementComposite,
		//	composite is empty
		if (!(featureComponent instanceof FeatureGroupComposite)) {
			return;
		}
		
		if (featureComponent.getChildren().length == 1 &&
				!(featureComponent.getChildren()[0] instanceof 
						DescriptionElementComposite)) {
			Composite grandParent = featureComponent.getParent(); 
			featureComponent.dispose();
			grandParent.layout();
		}

		managedForm.getForm().getBody().layout();
	}

	@Override
	public IPropertySource getPropertySource() {
		return new DescriptionElementPropertySource((DescriptionElementBase) getData());
	}

	@Override
	protected Font getViewerFont() {
		// TODO Auto-generated method stub
		return null;
	}
}