/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.editor.name;

import java.util.Iterator;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IWorkspaceRunnable;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.Assert;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.text.Document;
import org.eclipse.jface.text.ILineTracker;
import org.eclipse.jface.text.IUndoManager;
import org.eclipse.jface.text.IUndoManagerExtension;
import org.eclipse.jface.text.Position;
import org.eclipse.jface.text.TextViewerUndoManager;
import org.eclipse.jface.text.source.Annotation;
import org.eclipse.jface.text.source.AnnotationModel;
import org.eclipse.jface.text.source.AnnotationPainter;
import org.eclipse.jface.text.source.IAnnotationAccess;
import org.eclipse.jface.text.source.SourceViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.actions.ActionFactory;
import org.eclipse.ui.forms.widgets.TableWrapData;
import org.eclipse.ui.operations.OperationHistoryActionHandler;
import org.eclipse.ui.operations.RedoActionHandler;
import org.eclipse.ui.operations.UndoActionHandler;
import org.eclipse.ui.texteditor.IWorkbenchActionDefinitionIds;

import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.taxeditor.editor.AnnotationMarkerAccess;
import eu.etaxonomy.taxeditor.editor.ErrorAnnotation;
import eu.etaxonomy.taxeditor.editor.LineBreakListener;
import eu.etaxonomy.taxeditor.editor.LineWrapSquigglesStrategy;
import eu.etaxonomy.taxeditor.editor.ParseListener;
import eu.etaxonomy.taxeditor.editor.WarningAnnotation;


/**
 * SourceViewer implementation called by NameComposite.
 *  
 * @author p.ciardelli
 * @created 27.05.2008
 * @version 1.0
 */
public class NameViewer extends SourceViewer {
	private static final Logger logger = Logger
			.getLogger(NameViewer.class);

	public NameViewer(Composite parent) {
		super(parent, null, SWT.WRAP | SWT.MULTI | SWT.RESIZE);
							
		// Set name viewer's text to name cache
		AnnotationModel model = new AnnotationModel();
		NameViewerDocument document = new NameViewerDocument("");
		this.setDocument(document, model);
//		model.connect(document);

		// Lay out StyledText
		StyledText textWidget = this.getTextWidget();
		textWidget.setLayoutData(new TableWrapData(TableWrapData.FILL_GRAB));
		
		// Annotations section
		IAnnotationAccess fAnnotationAccess = new AnnotationMarkerAccess();
		
		// To paint the annotations
		AnnotationPainter annotationPainter = new AnnotationPainter(this, fAnnotationAccess);

		// Default SquigglesStrategy doesn't recognize line wraps
		annotationPainter.addDrawingStrategy(LineWrapSquigglesStrategy.ID, new LineWrapSquigglesStrategy());
		
		// Add ability to paint red squigglies
		annotationPainter.addAnnotationType(ErrorAnnotation.ERROR_TYPE, LineWrapSquigglesStrategy.ID);
		annotationPainter.setAnnotationTypeColor(ErrorAnnotation.ERROR_TYPE, 
				new Color(Display.getDefault(), ErrorAnnotation.ERROR_RGB));

		// Add ability to paint yellow squigglies
		annotationPainter.addAnnotationType(WarningAnnotation.WARNING_TYPE, LineWrapSquigglesStrategy.ID);
		annotationPainter.setAnnotationTypeColor(WarningAnnotation.WARNING_TYPE, 
				new Color(Display.getDefault(), WarningAnnotation.WARNING_RGB));		
				
		this.addPainter(annotationPainter);
		
//		DefaultAnnotationHover hover = new DefaultAnnotationHover();
//				
//		CompositeRuler fCompositeRuler = new CompositeRuler();
//		AnnotationRulerColumn annotationCol = new AnnotationRulerColumn(model, 10);
//		AnnotationBarHoverManager fAnnotationHoverManager = new AnnotationBarHoverManager(annotationCol, this, 
//				new DefaultAnnotationHover(), fHoverControlCreator);
//		fAnnotationHoverManager.install(annotationRuler.getControl());

		
//		DocumentUndoManagerRegistry.connect(this.getDocument());
//		IDocumentUndoManager docUndoManager = DocumentUndoManagerRegistry.getDocumentUndoManager(this.getDocument());
		
		/**
		getEditorSite().getActionBars().setGlobalActionHandler(ActionFactory.UNDO.getId(), new UndoActionHandler(getSite(), undoContext));
		getEditorSite().getActionBars().setGlobalActionHandler(ActionFactory.REDO.getId(), new RedoActionHandler(getSite(), undoContext));		
		**/
		

	}
	
	/**
	 * If <code>hasProblem</code> is <code>true</code>, underlines 
	 * entire contents of text widget in red squigglies.
	 * 
	 * @param hasProblem
	 */
	public void setShowError(boolean hasProblem) {
		
		String text = this.getTextWidget().getText();
				
		Iterator<Annotation> annotations = this.getAnnotationModel().getAnnotationIterator();
		while (annotations.hasNext()) {
			Annotation annotation = annotations.next(); 
			if (annotation.getType().equals(ErrorAnnotation.ERROR_TYPE))
				this.getAnnotationModel().removeAnnotation(annotation);
		}
		
		if (hasProblem && text.length() > 0) {
			this.getAnnotationModel().addAnnotation(
					new ErrorAnnotation(0, text), 
					new Position(0, text.length()));				
		}
	}

	/**
	 * If <code>name.hasProblem()</code> is <code>true</code>, underlines section
	 * of text bounded by <code>name.getProblemStarts()</code> and 
	 * <code>name.getProblemEnds()</code>.
	 * 
	 * @param name
	 */
	public void setShowError(TaxonNameBase name) {
		
		boolean hasProblem = name.getHasProblem();
		
		if (!hasProblem) {
			logger.warn(name.getProblemStarts());
		}
		
		String text = this.getTextWidget().getText();
				
		Iterator<Annotation> annotations = this.getAnnotationModel().getAnnotationIterator();
		while (annotations.hasNext()) {
			Annotation annotation = annotations.next(); 
			if (annotation.getType().equals(ErrorAnnotation.ERROR_TYPE))
				this.getAnnotationModel().removeAnnotation(annotation);
		}
		
		if (hasProblem && text.length() > 0) {
//			logger.warn("Name: " + name.getTitleCache() + ", start: " + name.getProblemStarts()+ ", end: " + name.getProblemEnds());
			int start = name.getProblemStarts();
			int length = name.getProblemEnds() - start;
			
			if (start == -1 || name.getProblemEnds() == -1) {
//				logger.warn("Start: " + start + ", End " + name.getProblemEnds());
				return;
			}
			
			// Don't let squigglies try to draw beyond the end of the text
			if (text.length() < start + length) {
				length = text.length() - start;
			}
			
			this.getAnnotationModel().addAnnotation(
					new ErrorAnnotation(0, text), 
					new Position(start, length));
			
			// Successful test of markers which show up in ProblemView 
//			try {
//				IWorkspaceRunnable editorMarker = new IWorkspaceRunnable() {
//					public void run(IProgressMonitor monitor) throws CoreException { 
//						IMarker marker = ResourcesPlugin.getWorkspace().getRoot().createMarker(IMarker.PROBLEM);
//						marker.setAttribute(IMarker.MESSAGE, "Testeroo");
//						marker.setAttribute(IMarker.PRIORITY, IMarker.PRIORITY_HIGH);
//						marker.setAttribute(IMarker.LINE_NUMBER, 1);
//						marker.setAttribute(IMarker.TRANSIENT, false);
//						marker.setAttribute(IMarker.SEVERITY, IMarker.SEVERITY_ERROR); 						
//					}
//				};
//				ResourcesPlugin.getWorkspace().run(editorMarker, null); 
//				
//			} catch (CoreException e) {
//				// TODO Auto-generated catch block
//				e.printStackTrace();
//			}
			
		}
	}
		
	public void addLineBreakListener(LineBreakListener lineBreakListener) {
		this.getTextWidget().addVerifyListener(lineBreakListener);
	}
	
	public void removeLineBreakListener(LineBreakListener lineBreakListener) {
		this.getTextWidget().removeVerifyListener(lineBreakListener);
	}
	
	public void addParseListener(ParseListener parseListener) {
		this.getTextWidget().addModifyListener(parseListener);
	}

	public void removeParseListener(ParseListener parseListener) {
		this.getTextWidget().removeModifyListener(parseListener);
	}
	
	public void setText(String text) {
		if (text == null) {
			text = "";
		}
		try {
			Assert.isNotNull(text);
			// TODO figure out why getTextWidget() returns null!
			if (this.getTextWidget() == null) {
				return;
			}
			Assert.isNotNull(this.getTextWidget());
			this.getTextWidget().setText(text);
		} catch (RuntimeException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public void setCursorToEOL() {
		this.getTextWidget().setCaretOffset(getTextWidget().getText().length());
	}
	
	class NameViewerDocument extends Document {
		public NameViewerDocument(String string) {
			super(string);
		}

		public ILineTracker getTracker() {
			return super.getTracker();
		}	
	}

	public void createUndoSupport(IEditorSite editorSite) {

		IUndoManager undoManager = new TextViewerUndoManager(25);
		this.setUndoManager(undoManager);
		undoManager.connect(this);
		
//		IUndoContext workbenchUndoContext = UiUtil.getWorkbenchUndoContext();
		
		IUndoContext workbenchUndoContext = ((IUndoManagerExtension)undoManager).getUndoContext();
		
		OperationHistoryActionHandler undoAction = new UndoActionHandler(editorSite, workbenchUndoContext);
//		undoAction.setActionDefinitionId(IWorkbenchActionDefinitionIds.UNDO);
		undoAction.setActionDefinitionId(ActionFactory.UNDO.getId());
		
		// Create the redo action.
		OperationHistoryActionHandler redoAction = new RedoActionHandler(editorSite, workbenchUndoContext);
//		redoAction.setActionDefinitionId(IWorkbenchActionDefinitionIds.REDO);
		undoAction.setActionDefinitionId(ActionFactory.REDO.getId());

		IActionBars actionBars = editorSite.getActionBars();
		if (actionBars != null) {
//				actionBars.setGlobalActionHandler(IWorkbenchActionDefinitionIds.UNDO, undoAction);
//				actionBars.setGlobalActionHandler(IWorkbenchActionDefinitionIds.REDO, redoAction);
			actionBars.setGlobalActionHandler(ActionFactory.UNDO.getId(), undoAction);
			actionBars.setGlobalActionHandler(ActionFactory.REDO.getId(), redoAction);
			
		}		
		
//		actionBars.setGlobalActionHandler(ActionFactory.UNDO.getId(), undoAction);
//		actionBars.setGlobalActionHandler(ActionFactory.REDO.getId(), redoAction);
	}
	
	public void createUndoSupport_(IEditorSite editorSite) {
		IUndoManager undoManager = new TextViewerUndoManager(25);
		this.setUndoManager(undoManager);
		undoManager.connect(this);
		IUndoContext undoContext;
		if (undoManager instanceof IUndoManagerExtension) {
			undoContext = ((IUndoManagerExtension)undoManager).getUndoContext();
		
			OperationHistoryActionHandler undoAction = new UndoActionHandler(editorSite, undoContext);
//			PlatformUI.getWorkbench().getHelpSystem().setHelp(undoAction, IAbstractTextEditorHelpContextIds.UNDO_ACTION);
			undoAction.setActionDefinitionId(IWorkbenchActionDefinitionIds.UNDO);
			
			// Create the redo action.
			OperationHistoryActionHandler redoAction = new RedoActionHandler(editorSite, undoContext);
//			PlatformUI.getWorkbench().getHelpSystem().setHelp(redoAction, IAbstractTextEditorHelpContextIds.REDO_ACTION);
			redoAction.setActionDefinitionId(IWorkbenchActionDefinitionIds.REDO);


			IActionBars actionBars = editorSite.getActionBars();
			if (actionBars != null) {
				actionBars.setGlobalActionHandler(IWorkbenchActionDefinitionIds.UNDO, undoAction);
				actionBars.setGlobalActionHandler(IWorkbenchActionDefinitionIds.REDO, redoAction);
				
			}
		}
	}
	
//	public boolean isMultiLine(Annotation annotation) {
//	    return true;
//	}
}
