/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.propertysheet.namerelationswizard;

import org.apache.log4j.Logger;
import org.eclipse.core.databinding.beans.BeansObservables;
import org.eclipse.core.databinding.observable.list.WritableList;
import org.eclipse.core.databinding.observable.map.IObservableMap;
import org.eclipse.jface.databinding.viewers.ObservableListContentProvider;
import org.eclipse.jface.databinding.viewers.ObservableMapLabelProvider;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableItem;

import eu.etaxonomy.cdm.model.name.NameRelationship;
import eu.etaxonomy.cdm.model.name.NameRelationshipType;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.name.ZoologicalName;
import eu.etaxonomy.taxeditor.ITaxEditorConstants;
import eu.etaxonomy.taxeditor.TaxEditorPlugin;
import eu.etaxonomy.taxeditor.actions.cdm.DeleteNameRelationAction;
import eu.etaxonomy.taxeditor.actions.ui.OpenNameRelationWizardAction;
import eu.etaxonomy.taxeditor.controller.GlobalController;
import eu.etaxonomy.taxeditor.model.CdmUtil;

/**
 * @author p.ciardelli
 * @created 06.06.2008
 * @version 1.0
 */
public class ListNameRelationsWizardPage extends WizardPage {
	private static final Logger logger = Logger
			.getLogger(ListNameRelationsWizardPage.class);

	private Table nameRelationsTable;
	private WritableList nameRelationsList = new WritableList();
	private TaxonNameBase name;

	private Button btnRemove;
	private Button btnEdit;

	private NameRelationship selectedRelation;

	private boolean isZoological = false;

	public ListNameRelationsWizardPage(TaxonNameBase name) {
		super("");

		this.name = name;
		if (name instanceof ZoologicalName) {
			isZoological  = true;
		}

		this.nameRelationsList.addAll(name.getNameRelations());

		setTitle("Nomenclatural relations for \""
				+ CdmUtil.getDisplayName(name) + "\".");
		setDescription("Select a relation and click \"Edit ...\" or \"Remove\", or click \"Add ...\" to create a new relation.");
	}

	public void createControl(Composite parent) {
		Composite container = new Composite(parent, SWT.NULL);
		final GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 3;
		container.setLayout(gridLayout);

		setControl(container);

		btnEdit = new Button(container, SWT.NONE);
		btnEdit.setText("Edit ...");

		btnRemove = new Button(container, SWT.NONE);
		btnRemove.setText("Remove");
		btnRemove.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				if (getSelectedRelation() != null) {
					new DeleteNameRelationAction(name, getSelectedRelation()).run();
					nameRelationsList.remove(getSelectedRelation());
					setSelectedRelation(null);
				}
			}
		});

		setEnableExistingRelationButtons(false);

		final Button btnAdd = new Button(container, SWT.NONE);
		final GridData gd_btnAdd = new GridData(SWT.RIGHT, SWT.CENTER, true,
				false);
		btnAdd.setLayoutData(gd_btnAdd);
		btnAdd.setText("Add ...");
		btnAdd.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				createRelationWizard(null);
			}
		});

		final TableViewer tableViewer = new TableViewer(container, SWT.BORDER | SWT.SINGLE);

		nameRelationsTable = tableViewer.getTable();
		nameRelationsTable.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true,
				true, 3, 1));
		nameRelationsTable.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				// TODO: As long as onNameBase.removeNameRelationship() doesn't work, "Remove" stays disabled ... 
				setEnableExistingRelationButtons(true);
				TableItem[] selectedItem = nameRelationsTable.getSelection();
				if (e.item.getData() instanceof NameRelationship) {
					setSelectedRelation((NameRelationship) e.item.getData());
				}
			}
		});

		ObservableListContentProvider providerList = new ObservableListContentProvider();
		tableViewer.setContentProvider(providerList);

		IObservableMap[] providerMaps = BeansObservables.observeMaps(
				providerList.getKnownElements(), NameRelationship.class,
				new String[] { "fromName", "toName", "type" });
		tableViewer.setLabelProvider(new ObservableMapLabelProvider(
				providerMaps) {
			public String getColumnText(Object element, int columnIndex) {
				if (element instanceof NameRelationship) {
					NameRelationship nameRelationship = (NameRelationship) element;
					return getRelationshipString(nameRelationship);
				}
				return "";
			}

			public Image getColumnImage(Object element, int columnIndex) {
				if (element instanceof NameRelationship) {
					NameRelationship nameRelationship = (NameRelationship) element;
					return getRelationshipImage(nameRelationship);
				}
				return null;
			}
		});
		// TODO try name.getNameRelations()
		tableViewer.setInput(nameRelationsList);
		tableViewer.addDoubleClickListener(new IDoubleClickListener() {
			public void doubleClick(DoubleClickEvent event) {
				if (((StructuredSelection) event.getSelection())
						.getFirstElement() instanceof NameRelationship) {
					NameRelationship relation = (NameRelationship) ((StructuredSelection) event
							.getSelection()).getFirstElement();
					createRelationWizard(relation);
				}
			}
		});
	}

	private void setSelectedRelation(NameRelationship selectedRelation) {
		this.selectedRelation = selectedRelation;
	}

	private NameRelationship getSelectedRelation() {
		return selectedRelation;
	}
	
	private void createRelationWizard(NameRelationship relation) {
		if (relation == null) {
			new OpenNameRelationWizardAction(name, nameRelationsList).run();
		} else {
			new OpenNameRelationWizardAction(name, relation).run();
		}
	}

	protected void setEnableExistingRelationButtons(boolean enabled) {
		btnRemove.setEnabled(enabled);
		btnEdit.setEnabled(false);
	}

	private Image getRelationshipImage(NameRelationship nameRelationship) {
		if (nameRelationship.getType().equals(NameRelationshipType.BASIONYM())) {
			if (isZoological) { 
				return null;
			}
			return TaxEditorPlugin.getDefault().getImage(
					ITaxEditorConstants.BASIONYM_ICON);
		}
		if (nameRelationship.getType().equals(
				NameRelationshipType.ORTHOGRAPHIC_VARIANT())) {
			return TaxEditorPlugin.getDefault().getImage(
					ITaxEditorConstants.ORTHOGRAPHIC_VARIANT_ICON);
		}
//		return TaxEditorPlugin.getDefault().getImage(
//				ITaxEditorConstants.EDIT_ICON);
		return null;
	}

	private String getRelationshipString(NameRelationship nameRelationship) {
		String fromName = null;
		String type = null;
		String toName = null;
		if (nameRelationship.getFromName() == null) {
			fromName = "";
		} else {
			fromName = CdmUtil.getDisplayName(nameRelationship.getFromName());
		}
		if (nameRelationship.getType() == null) {
			type = "";
		} else {
			if (isZoological && 
					nameRelationship.getType().equals(NameRelationshipType.BASIONYM())) {
				type = "original combination";
			} else {
//				type = nameRelationship.getType().getLabel();
				type = GlobalController.getNameRelationLabelType(nameRelationship.getType());
			}
		}
		if (nameRelationship.getToName() == null) {
			toName = "";
		} else {
			toName = CdmUtil.getDisplayName(nameRelationship.getToName());
		}
		return "\"" + fromName + "\" is \"" + type + "\" of \"" + toName + "\"";
	}
}
